@@ -3,6 +3,7 @@ package chezmoi
 import (
 	"io/fs"
 	"os/exec"
+	"time"
 
 	vfs "github.com/twpayne/go-vfs/v4"
 )
@@ -27,6 +28,12 @@ func (s *DryRunSystem) Chmod(name AbsPath, mode fs.FileMode) error {
 	return nil
 }
 
+// Chtimes implements System.Chtimes.
+func (s *DryRunSystem) Chtimes(name AbsPath, atime, mtime time.Time) error {
+	s.setModified()
+	return nil
+}
+
 // Glob implements System.Glob.
 func (s *DryRunSystem) Glob(pattern string) ([]string, error) {
 	return s.system.Glob(pattern)
