@@ -15,7 +15,7 @@ import (
 )
 
 // A TextConvFunc converts the contents of a file into a more human-readable form.
-type TextConvFunc func(AbsPath, []byte) ([]byte, error)
+type TextConvFunc func(string, []byte) ([]byte, error)
 
 // A GitDiffSystem wraps a System and logs all of the actions executed as a git
 // diff.
@@ -267,7 +267,7 @@ func (s *GitDiffSystem) encodeDiff(absPath AbsPath, toData []byte, toMode fs.Fil
 			return err
 		}
 		if s.textConvFunc != nil {
-			fromData, err = s.textConvFunc(absPath, fromData)
+			fromData, err = s.textConvFunc(absPath.String(), fromData)
 			if err != nil {
 				return err
 			}
@@ -286,7 +286,7 @@ func (s *GitDiffSystem) encodeDiff(absPath AbsPath, toData []byte, toMode fs.Fil
 
 	if s.textConvFunc != nil {
 		var err error
-		toData, err = s.textConvFunc(absPath, toData)
+		toData, err = s.textConvFunc(absPath.String(), toData)
 		if err != nil {
 			return err
 		}
