@@ -37,11 +37,11 @@ import (
 	"github.com/rs/zerolog/log"
 	"github.com/spf13/cobra"
 	"github.com/spf13/pflag"
-	"github.com/twpayne/go-shell"
 	"github.com/twpayne/go-vfs/v4"
 	"github.com/twpayne/go-xdg/v6"
 	cobracompletefig "github.com/withfig/autocomplete-tools/integrations/cobra"
 	"go.uber.org/multierr"
+	"golang.org/x/exp/maps"
 	"golang.org/x/term"
 	"mvdan.cc/sh/v3/expand"
 	"mvdan.cc/sh/v3/syntax"
@@ -50,6 +50,7 @@ import (
 	"github.com/twpayne/chezmoi/v2/pkg/chezmoi"
 	"github.com/twpayne/chezmoi/v2/pkg/chezmoilog"
 	"github.com/twpayne/chezmoi/v2/pkg/git"
+	"github.com/twpayne/chezmoi/v2/pkg/shell"
 )
 
 const (
@@ -86,6 +87,7 @@ type ConfigFile struct {
 	PINEntry           pinEntryConfig                  `mapstructure:"pinentry"`
 	Progress           bool                            `mapstructure:"progress"`
 	Safe               bool                            `mapstructure:"safe"`
+	ScriptEnv          map[string]string               `mapstructure:"scriptEnv"`
 	ScriptTempDir      chezmoi.AbsPath                 `mapstructure:"scriptTempDir"`
 	SourceDirAbsPath   chezmoi.AbsPath                 `mapstructure:"sourceDir"`
 	Template           templateConfig                  `mapstructure:"template"`
@@ -177,6 +179,7 @@ type Config struct {
 	fileSystem             vfs.FS
 	bds                    *xdg.BaseDirectorySpecification
 	configFileAbsPath      chezmoi.AbsPath
+	configFileAbsPathErr   error
 	baseSystem             chezmoi.System
 	sourceSystem           chezmoi.System
 	destSystem             chezmoi.System
@@ -186,10 +189,13 @@ type Config struct {
 	logger                 *zerolog.Logger
 
 	// Computed configuration.
-	homeDirAbsPath chezmoi.AbsPath
-	encryption     chezmoi.Encryption
-	sourceState    *chezmoi.SourceState
-	sourceStateErr error
+	homeDirAbsPath      chezmoi.AbsPath
+	encryption          chezmoi.Encryption
+	sourceDirAbsPath    chezmoi.AbsPath
+	sourceDirAbsPathErr error
+	sourceState         *chezmoi.SourceState
+	sourceStateErr      error
+	templateData        *templateData
 
 	stdin       io.Reader
 	stdout      io.Writer
@@ -203,6 +209,28 @@ type Config struct {
 	restoreWindowsConsole func() error
 }
 
+type templateData struct {
+	Arch           string          `json:"arch"`
+	Args           []string        `json:"args"`
+	CacheDir       chezmoi.AbsPath `json:"cacheDir"`
+	ConfigFile     chezmoi.AbsPath `json:"configFile"`
+	Executable     chezmoi.AbsPath `json:"executable"`
+	FQDNHostname   string          `json:"fqdnHostname"`
+	GID            string          `json:"gid"`
+	Group          string          `json:"group"`
+	HomeDir        chezmoi.AbsPath `json:"homeDir"`
+	Hostname       string          `json:"hostname"`
+	Kernel         map[string]any  `json:"kernel"`
+	OS             string          `json:"os"`
+	OSRelease      map[string]any  `json:"osRelease"`
+	SourceDir      chezmoi.AbsPath `json:"sourceDir"`
+	UID            string          `json:"uid"`
+	Username       string          `json:"username"`
+	Version        map[string]any  `json:"version"`
+	WindowsVersion map[string]any  `json:"windowsVersion"`
+	WorkingTree    chezmoi.AbsPath `json:"workingTree"`
+}
+
 // A configOption sets and option on a Config.
 type configOption func(*Config) error
 
@@ -328,6 +356,7 @@ func newConfig(options ...configOption) (*Config, error) {
 		"comment":                  c.commentTemplateFunc,
 		"decrypt":                  c.decryptTemplateFunc,
 		"encrypt":                  c.encryptTemplateFunc,
+		"eqFold":                   c.eqFoldTemplateFunc,
 		"fromIni":                  c.fromIniTemplateFunc,
 		"fromToml":                 c.fromTomlTemplateFunc,
 		"fromYaml":                 c.fromYamlTemplateFunc,
@@ -385,10 +414,7 @@ func newConfig(options ...configOption) (*Config, error) {
 	if err != nil {
 		return nil, err
 	}
-	c.configFileAbsPath, err = c.defaultConfigFile(c.fileSystem, c.bds)
-	if err != nil {
-		return nil, err
-	}
+	c.configFileAbsPath, c.configFileAbsPathErr = c.defaultConfigFile(c.fileSystem, c.bds)
 	c.SourceDirAbsPath, err = c.defaultSourceDir(c.fileSystem, c.bds)
 	if err != nil {
 		return nil, err
@@ -413,7 +439,7 @@ func (c *Config) Close() error {
 	return err
 }
 
-// addTemplateFunc adds the template function with the key key and value value
+// addTemplateFunc adds the template function with the given key and value
 // to c. It panics if there is already an existing template function with the
 // same key.
 func (c *Config) addTemplateFunc(key string, value any) {
@@ -447,7 +473,7 @@ func (c *Config) applyArgs(
 	}
 
 	var currentConfigTemplateContentsSHA256 []byte
-	configTemplate, err := c.findFirstConfigTemplate()
+	configTemplate, err := c.findConfigTemplate()
 	if err != nil {
 		return err
 	}
@@ -602,7 +628,7 @@ func (c *Config) colorAutoFunc() bool {
 // template and reloads it.
 func (c *Config) createAndReloadConfigFile() error {
 	// Find config template, execute it, and create config file.
-	configTemplate, err := c.findFirstConfigTemplate()
+	configTemplate, err := c.findConfigTemplate()
 	if err != nil {
 		return err
 	}
@@ -682,7 +708,7 @@ func (c *Config) createConfigFile(filename chezmoi.RelPath, data []byte) ([]byte
 	}
 
 	builder := strings.Builder{}
-	templateData := c.defaultTemplateData()
+	templateData := c.getTemplateDataMap()
 	if c.init.data {
 		chezmoi.RecursiveMerge(templateData, c.Data)
 	}
@@ -694,22 +720,51 @@ func (c *Config) createConfigFile(filename chezmoi.RelPath, data []byte) ([]byte
 
 // defaultConfigFile returns the default config file according to the XDG Base
 // Directory Specification.
-func (c *Config) defaultConfigFile(
-	fileSystem vfs.Stater, bds *xdg.BaseDirectorySpecification,
-) (chezmoi.AbsPath, error) {
+func (c *Config) defaultConfigFile(fileSystem vfs.FS, bds *xdg.BaseDirectorySpecification) (chezmoi.AbsPath, error) {
 	// Search XDG Base Directory Specification config directories first.
+CONFIG_DIR:
 	for _, configDir := range bds.ConfigDirs {
 		configDirAbsPath, err := chezmoi.NewAbsPathFromExtPath(configDir, c.homeDirAbsPath)
 		if err != nil {
 			return chezmoi.EmptyAbsPath, err
 		}
+
+		dirEntries, err := fileSystem.ReadDir(configDirAbsPath.JoinString("chezmoi").String())
+		switch {
+		case errors.Is(err, fs.ErrNotExist):
+			continue CONFIG_DIR
+		case err != nil:
+			return chezmoi.EmptyAbsPath, err
+		}
+
+		dirEntryNames := make(map[string]struct{}, len(dirEntries))
+		for _, dirEntry := range dirEntries {
+			dirEntryNames[dirEntry.Name()] = struct{}{}
+		}
+
+		var names []string
 		for _, extension := range chezmoi.FormatExtensions {
-			configFileAbsPath := configDirAbsPath.JoinString("chezmoi", "chezmoi."+extension)
-			if _, err := fileSystem.Stat(configFileAbsPath.String()); err == nil {
-				return configFileAbsPath, nil
+			name := "chezmoi." + extension
+			if _, ok := dirEntryNames[name]; ok {
+				names = append(names, name)
 			}
 		}
+
+		switch len(names) {
+		case 0:
+			// Do nothing.
+		case 1:
+			return configDirAbsPath.JoinString("chezmoi", names[0]), nil
+		default:
+			configFileAbsPathStrs := make([]string, 0, len(names))
+			for _, name := range names {
+				configFileAbsPathStr := configDirAbsPath.JoinString("chezmoi", name)
+				configFileAbsPathStrs = append(configFileAbsPathStrs, configFileAbsPathStr.String())
+			}
+			return chezmoi.EmptyAbsPath, fmt.Errorf("multiple config files: %s", englishList(configFileAbsPathStrs))
+		}
 	}
+
 	// Fallback to XDG Base Directory Specification default.
 	configHomeAbsPath, err := chezmoi.NewAbsPathFromExtPath(bds.ConfigHome, c.homeDirAbsPath)
 	if err != nil {
@@ -898,126 +953,6 @@ func (c *Config) defaultSourceDir(fileSystem vfs.Stater, bds *xdg.BaseDirectoryS
 	return dataHomeAbsPath.Join(chezmoiRelPath), nil
 }
 
-// defaultTemplateData returns the default template data.
-func (c *Config) defaultTemplateData() map[string]any {
-	// Determine the user's username and group, if possible.
-	//
-	// user.Current and user.LookupGroupId in Go's standard library are
-	// generally unreliable, so work around errors if possible, or ignore them.
-	//
-	// If CGO is disabled, then the Go standard library falls back to parsing
-	// /etc/passwd and /etc/group, which will return incorrect results without
-	// error if the system uses an alternative password database such as NIS or
-	// LDAP.
-	//
-	// If CGO is enabled then user.Current and user.LookupGroupId will use the
-	// underlying libc functions, namely getpwuid_r and getgrnam_r. If linked
-	// with glibc this will return the correct result. If linked with musl then
-	// they will use musl's implementation which, like Go's non-CGO
-	// implementation, also only parses /etc/passwd and /etc/group and so also
-	// returns incorrect results without error if NIS or LDAP are being used.
-	//
-	// On Windows, the user's group ID returned by user.Current() is an SID and
-	// no further useful lookup is possible with Go's standard library.
-	//
-	// Since neither the username nor the group are likely widely used in
-	// templates, leave these variables unset if their values cannot be
-	// determined. Unset variables will trigger template errors if used,
-	// alerting the user to the problem and allowing them to find alternative
-	// solutions.
-	var gid, group, uid, username string
-	if currentUser, err := user.Current(); err == nil {
-		gid = currentUser.Gid
-		uid = currentUser.Uid
-		username = currentUser.Username
-		if runtime.GOOS != "windows" {
-			if rawGroup, err := user.LookupGroupId(currentUser.Gid); err == nil {
-				group = rawGroup.Name
-			} else {
-				c.logger.Info().
-					Str("gid", currentUser.Gid).
-					Err(err).
-					Msg("user.LookupGroupId")
-			}
-		}
-	} else {
-		c.logger.Info().
-			Err(err).
-			Msg("user.Current")
-		var ok bool
-		username, ok = os.LookupEnv("USER")
-		if !ok {
-			c.logger.Info().
-				Str("key", "USER").
-				Bool("ok", ok).
-				Msg("os.LookupEnv")
-		}
-	}
-
-	fqdnHostname, err := chezmoi.FQDNHostname(c.fileSystem)
-	if err != nil {
-		c.logger.Info().
-			Err(err).
-			Msg("chezmoi.FQDNHostname")
-	}
-	hostname, _, _ := strings.Cut(fqdnHostname, ".")
-
-	kernel, err := chezmoi.Kernel(c.fileSystem)
-	if err != nil {
-		c.logger.Info().
-			Err(err).
-			Msg("chezmoi.Kernel")
-	}
-
-	var osRelease map[string]any
-	switch runtime.GOOS {
-	case "openbsd", "windows":
-		// Don't populate osRelease on OSes where /etc/os-release does not
-		// exist.
-	default:
-		if rawOSRelease, err := chezmoi.OSRelease(c.baseSystem); err == nil {
-			osRelease = upperSnakeCaseToCamelCaseMap(rawOSRelease)
-		} else {
-			c.logger.Info().
-				Err(err).
-				Msg("chezmoi.OSRelease")
-		}
-	}
-
-	executable, _ := os.Executable()
-
-	windowsVersion, _ := windowsVersion()
-
-	return map[string]any{
-		"chezmoi": map[string]any{
-			"arch":         runtime.GOARCH,
-			"args":         os.Args,
-			"cacheDir":     c.CacheDirAbsPath.String(),
-			"configFile":   c.configFileAbsPath.String(),
-			"executable":   executable,
-			"fqdnHostname": fqdnHostname,
-			"gid":          gid,
-			"group":        group,
-			"homeDir":      c.homeDir,
-			"hostname":     hostname,
-			"kernel":       kernel,
-			"os":           runtime.GOOS,
-			"osRelease":    osRelease,
-			"sourceDir":    c.SourceDirAbsPath.String(),
-			"uid":          uid,
-			"username":     username,
-			"version": map[string]any{
-				"builtBy": c.versionInfo.BuiltBy,
-				"commit":  c.versionInfo.Commit,
-				"date":    c.versionInfo.Date,
-				"version": c.versionInfo.Version,
-			},
-			"windowsVersion": windowsVersion,
-			"workingTree":    c.WorkingTreeAbsPath.String(),
-		},
-	}
-}
-
 type destAbsPathInfosOptions struct {
 	follow         bool
 	ignoreNotExist bool
@@ -1093,6 +1028,20 @@ func (c *Config) diffFile(
 	if color {
 		unifiedEncoder.SetColor(diff.NewColorConfig())
 	}
+	if fromMode.IsRegular() {
+		var err error
+		fromData, err = c.TextConv.convert(path.String(), fromData)
+		if err != nil {
+			return err
+		}
+	}
+	if toMode.IsRegular() {
+		var err error
+		toData, err = c.TextConv.convert(path.String(), toData)
+		if err != nil {
+			return err
+		}
+	}
 	diffPatch, err := chezmoi.DiffPatch(path, fromData, fromMode, toData, toMode)
 	if err != nil {
 		return err
@@ -1181,32 +1130,61 @@ type configTemplate struct {
 	contents      []byte
 }
 
-// findFirstConfigTemplate searches for a config template, returning the path,
-// format, and contents of the first one that it finds.
-func (c *Config) findFirstConfigTemplate() (*configTemplate, error) {
-	sourceDirAbsPath, err := c.sourceDirAbsPath()
+// findConfigTemplate searches for a config template, returning the path,
+// format, and contents. It returns an error if multiple config file templates
+// are found.
+func (c *Config) findConfigTemplate() (*configTemplate, error) {
+	sourceDirAbsPath, err := c.getSourceDirAbsPath(nil)
 	if err != nil {
 		return nil, err
 	}
 
+	dirEntries, err := c.baseSystem.ReadDir(sourceDirAbsPath)
+	switch {
+	case errors.Is(err, fs.ErrNotExist):
+		return nil, nil
+	case err != nil:
+		return nil, err
+	}
+
+	dirEntryNames := make(map[chezmoi.RelPath]struct{}, len(dirEntries))
+	for _, dirEntry := range dirEntries {
+		dirEntryNames[chezmoi.NewRelPath(dirEntry.Name())] = struct{}{}
+	}
+
+	var configTemplates []*configTemplate //nolint:prealloc
 	for _, extension := range chezmoi.FormatExtensions {
 		relPath := chezmoi.NewRelPath(chezmoi.Prefix + "." + extension + chezmoi.TemplateSuffix)
+		if _, ok := dirEntryNames[relPath]; !ok {
+			continue
+		}
 		absPath := sourceDirAbsPath.Join(relPath)
 		contents, err := c.baseSystem.ReadFile(absPath)
-		switch {
-		case errors.Is(err, fs.ErrNotExist):
-			continue
-		case err != nil:
+		if err != nil {
 			return nil, err
 		}
-		return &configTemplate{
+		configTemplate := &configTemplate{
 			targetRelPath: chezmoi.NewRelPath("chezmoi." + extension),
 			format:        chezmoi.FormatsByExtension[extension],
 			sourceAbsPath: absPath,
 			contents:      contents,
-		}, nil
+		}
+		configTemplates = append(configTemplates, configTemplate)
+	}
+
+	switch len(configTemplates) {
+	case 0:
+		return nil, nil
+	case 1:
+		return configTemplates[0], nil
+	default:
+		sourceAbsPathStrs := make([]string, 0, len(configTemplates))
+		for _, configTemplate := range configTemplates {
+			sourceAbsPathStr := configTemplate.sourceAbsPath.String()
+			sourceAbsPathStrs = append(sourceAbsPathStrs, sourceAbsPathStr)
+		}
+		return nil, fmt.Errorf("multiple config file templates: %s ", englishList(sourceAbsPathStrs))
 	}
-	return nil, nil
 }
 
 func (c *Config) getHTTPClient() (*http.Client, error) {
@@ -1225,6 +1203,31 @@ func (c *Config) getHTTPClient() (*http.Client, error) {
 	return c.httpClient, nil
 }
 
+type getSourceDirAbsPathOptions struct {
+	refresh bool
+}
+
+// getSourceDirAbsPath returns the source directory, using .chezmoiroot if it
+// exists.
+func (c *Config) getSourceDirAbsPath(options *getSourceDirAbsPathOptions) (chezmoi.AbsPath, error) {
+	if options == nil || !options.refresh {
+		if !c.sourceDirAbsPath.Empty() || c.sourceDirAbsPathErr != nil {
+			return c.sourceDirAbsPath, c.sourceDirAbsPathErr
+		}
+	}
+
+	switch data, err := c.sourceSystem.ReadFile(c.SourceDirAbsPath.JoinString(chezmoi.RootName)); {
+	case errors.Is(err, fs.ErrNotExist):
+		c.sourceDirAbsPath = c.SourceDirAbsPath
+	case err != nil:
+		c.sourceDirAbsPathErr = err
+	default:
+		c.sourceDirAbsPath = c.SourceDirAbsPath.JoinString(string(bytes.TrimSpace(data)))
+	}
+
+	return c.sourceDirAbsPath, c.sourceDirAbsPathErr
+}
+
 func (c *Config) getSourceState(ctx context.Context) (*chezmoi.SourceState, error) {
 	if c.sourceState != nil || c.sourceStateErr != nil {
 		return c.sourceState, c.sourceStateErr
@@ -1233,6 +1236,31 @@ func (c *Config) getSourceState(ctx context.Context) (*chezmoi.SourceState, erro
 	return c.sourceState, c.sourceStateErr
 }
 
+// getTemplateData returns the default template data.
+func (c *Config) getTemplateData() *templateData {
+	if c.templateData == nil {
+		c.templateData = c.newTemplateData()
+	}
+	return c.templateData
+}
+
+// getTemplateDataMao returns the template data as a map.
+func (c *Config) getTemplateDataMap() map[string]any {
+	templateData := c.getTemplateData()
+	// FIXME round-tripping via JSON is a horrible hack
+	data, err := json.Marshal(templateData)
+	if err != nil {
+		panic(err)
+	}
+	var templateDataMap map[string]any
+	if err := json.Unmarshal(data, &templateDataMap); err != nil {
+		panic(err)
+	}
+	return map[string]any{
+		"chezmoi": templateDataMap,
+	}
+}
+
 // gitAutoAdd adds all changes to the git index and returns the new git status.
 func (c *Config) gitAutoAdd() (*git.Status, error) {
 	if err := c.run(c.WorkingTreeAbsPath, c.Git.Command, []string{"add", "."}); err != nil {
@@ -1255,7 +1283,11 @@ func (c *Config) gitAutoCommit(status *git.Status) error {
 	if err != nil {
 		return err
 	}
-	commitMessageTmpl, err := template.New("commit_message").Funcs(c.templateFuncs).Parse(string(commitMessageTemplate))
+	funcMap := maps.Clone(sprig.TxtFuncMap())
+	funcMap["targetRelPath"] = func(source string) string {
+		return chezmoi.NewSourceRelPath(source).TargetRelPath(c.encryption.EncryptedSuffix()).String()
+	}
+	commitMessageTmpl, err := template.New("commit_message").Funcs(funcMap).Parse(string(commitMessageTemplate))
 	if err != nil {
 		return err
 	}
@@ -1339,7 +1371,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 	persistentFlags.Lookup("refresh-externals").NoOptDefVal = chezmoi.RefreshExternalsAlways.String()
 	persistentFlags.BoolVar(&c.sourcePath, "source-path", c.sourcePath, "Specify targets by source path")
 
-	for _, err := range []error{
+	if err := multierr.Combine(
 		rootCmd.MarkPersistentFlagFilename("config"),
 		rootCmd.MarkPersistentFlagFilename("cpu-profile"),
 		persistentFlags.MarkHidden("cpu-profile"),
@@ -1353,10 +1385,8 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		rootCmd.RegisterFlagCompletionFunc("refresh-externals", chezmoi.RefreshExternalsFlagCompletionFunc),
 		rootCmd.RegisterFlagCompletionFunc("use-builtin-age", autoBoolFlagCompletionFunc),
 		rootCmd.RegisterFlagCompletionFunc("use-builtin-git", autoBoolFlagCompletionFunc),
-	} {
-		if err != nil {
-			return nil, err
-		}
+	); err != nil {
+		return nil, err
 	}
 
 	rootCmd.SetHelpCommand(c.newHelpCmd())
@@ -1444,7 +1474,7 @@ func (c *Config) newSourceState(
 
 	sourceStateLogger := c.logger.With().Str(logComponentKey, logComponentValueSourceState).Logger()
 
-	c.SourceDirAbsPath, err = c.sourceDirAbsPath()
+	c.SourceDirAbsPath, err = c.getSourceDirAbsPath(nil)
 	if err != nil {
 		return nil, err
 	}
@@ -1452,7 +1482,7 @@ func (c *Config) newSourceState(
 	sourceState := chezmoi.NewSourceState(append([]chezmoi.SourceStateOption{
 		chezmoi.WithBaseSystem(c.baseSystem),
 		chezmoi.WithCacheDir(c.CacheDirAbsPath),
-		chezmoi.WithDefaultTemplateDataFunc(c.defaultTemplateData),
+		chezmoi.WithDefaultTemplateDataFunc(c.getTemplateDataMap),
 		chezmoi.WithDestDir(c.DestDirAbsPath),
 		chezmoi.WithEncryption(c.encryption),
 		chezmoi.WithHTTPClient(httpClient),
@@ -1599,11 +1629,17 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	})
 
 	// Read the config file.
-	if err := c.readConfig(); err != nil {
-		if !boolAnnotation(cmd, doesNotRequireValidConfig) {
+	if boolAnnotation(cmd, doesNotRequireValidConfig) {
+		if c.configFileAbsPathErr == nil {
+			_ = c.readConfig()
+		}
+	} else {
+		if c.configFileAbsPathErr != nil {
+			return c.configFileAbsPathErr
+		}
+		if err := c.readConfig(); err != nil {
 			return fmt.Errorf("invalid config: %s: %w", c.configFileAbsPath, err)
 		}
-		c.errorf("warning: %s: %v\n", c.configFileAbsPath, err)
 	}
 
 	// Restore flags that were set on the command line.
@@ -1634,11 +1670,11 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 		Strs("args", os.Args).
 		Str("goVersion", runtime.Version()).
 		Msg("persistentPreRunRootE")
-
-	c.baseSystem = chezmoi.NewRealSystem(c.fileSystem,
+	realSystem := chezmoi.NewRealSystem(c.fileSystem,
 		chezmoi.RealSystemWithSafe(c.Safe),
 		chezmoi.RealSystemWithScriptTempDir(c.ScriptTempDir),
 	)
+	c.baseSystem = realSystem
 	if c.debug {
 		systemLogger := c.logger.With().Str(logComponentKey, logComponentValueSystem).Logger()
 		c.baseSystem = chezmoi.NewDebugSystem(c.baseSystem, &systemLogger)
@@ -1791,6 +1827,44 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 		}
 	}
 
+	scriptEnv := os.Environ()
+	templateData := c.getTemplateData()
+	for key, value := range map[string]string{
+		"ARCH":          templateData.Arch,
+		"ARGS":          strings.Join(templateData.Args, " "),
+		"CACHE_DIR":     templateData.CacheDir.String(),
+		"CONFIG_FILE":   templateData.ConfigFile.String(),
+		"EXECUTABLE":    templateData.Executable.String(),
+		"FQDN_HOSTNAME": templateData.FQDNHostname,
+		"GID":           templateData.GID,
+		"GROUP":         templateData.Group,
+		"HOME_DIR":      templateData.HomeDir.String(),
+		"HOSTNAME":      templateData.Hostname,
+		"OS":            templateData.OS,
+		"SOURCE_DIR":    templateData.SourceDir.String(),
+		"UID":           templateData.UID,
+		"USERNAME":      templateData.Username,
+		"WORKING_TREE":  templateData.WorkingTree.String(),
+	} {
+		scriptEnv = append(scriptEnv, "CHEZMOI_"+key+"="+value)
+	}
+	for groupKey, group := range map[string]map[string]any{
+		"KERNEL":          templateData.Kernel,
+		"OS_RELEASE":      templateData.OSRelease,
+		"VERSION":         templateData.Version,
+		"WINDOWS_VERSION": templateData.WindowsVersion,
+	} {
+		for key, value := range group {
+			upperSnakeCaseKey := camelCaseToUpperSnakeCase(key)
+			valueStr := fmt.Sprintf("%s", value)
+			scriptEnv = append(scriptEnv, "CHEZMOI_"+groupKey+"_"+upperSnakeCaseKey+"="+valueStr)
+		}
+	}
+	for key, value := range c.ScriptEnv {
+		scriptEnv = append(scriptEnv, key+"="+value)
+	}
+	realSystem.SetScriptEnv(scriptEnv)
+
 	return nil
 }
 
@@ -1821,6 +1895,123 @@ func (c *Config) persistentStateFile() (chezmoi.AbsPath, error) {
 	return defaultConfigFileAbsPath.Dir().Join(persistentStateFileRelPath), nil
 }
 
+func (c *Config) newTemplateData() *templateData {
+	// Determine the user's username and group, if possible.
+	//
+	// user.Current and user.LookupGroupId in Go's standard library are
+	// generally unreliable, so work around errors if possible, or ignore them.
+	//
+	// If CGO is disabled, then the Go standard library falls back to parsing
+	// /etc/passwd and /etc/group, which will return incorrect results without
+	// error if the system uses an alternative password database such as NIS or
+	// LDAP.
+	//
+	// If CGO is enabled then user.Current and user.LookupGroupId will use the
+	// underlying libc functions, namely getpwuid_r and getgrnam_r. If linked
+	// with glibc this will return the correct result. If linked with musl then
+	// they will use musl's implementation which, like Go's non-CGO
+	// implementation, also only parses /etc/passwd and /etc/group and so also
+	// returns incorrect results without error if NIS or LDAP are being used.
+	//
+	// On Windows, the user's group ID returned by user.Current() is an SID and
+	// no further useful lookup is possible with Go's standard library.
+	//
+	// Since neither the username nor the group are likely widely used in
+	// templates, leave these variables unset if their values cannot be
+	// determined. Unset variables will trigger template errors if used,
+	// alerting the user to the problem and allowing them to find alternative
+	// solutions.
+	var gid, group, uid, username string
+	if currentUser, err := user.Current(); err == nil {
+		gid = currentUser.Gid
+		uid = currentUser.Uid
+		username = currentUser.Username
+		if runtime.GOOS != "windows" {
+			if rawGroup, err := user.LookupGroupId(currentUser.Gid); err == nil {
+				group = rawGroup.Name
+			} else {
+				c.logger.Info().
+					Str("gid", currentUser.Gid).
+					Err(err).
+					Msg("user.LookupGroupId")
+			}
+		}
+	} else {
+		c.logger.Info().
+			Err(err).
+			Msg("user.Current")
+		var ok bool
+		username, ok = os.LookupEnv("USER")
+		if !ok {
+			c.logger.Info().
+				Str("key", "USER").
+				Bool("ok", ok).
+				Msg("os.LookupEnv")
+		}
+	}
+
+	fqdnHostname, err := chezmoi.FQDNHostname(c.fileSystem)
+	if err != nil {
+		c.logger.Info().
+			Err(err).
+			Msg("chezmoi.FQDNHostname")
+	}
+	hostname, _, _ := strings.Cut(fqdnHostname, ".")
+
+	kernel, err := chezmoi.Kernel(c.fileSystem)
+	if err != nil {
+		c.logger.Info().
+			Err(err).
+			Msg("chezmoi.Kernel")
+	}
+
+	var osRelease map[string]any
+	switch runtime.GOOS {
+	case "openbsd", "windows":
+		// Don't populate osRelease on OSes where /etc/os-release does not
+		// exist.
+	default:
+		if rawOSRelease, err := chezmoi.OSRelease(c.fileSystem); err == nil {
+			osRelease = upperSnakeCaseToCamelCaseMap(rawOSRelease)
+		} else {
+			c.logger.Info().
+				Err(err).
+				Msg("chezmoi.OSRelease")
+		}
+	}
+
+	executable, _ := os.Executable()
+	windowsVersion, _ := windowsVersion()
+	sourceDirAbsPath, _ := c.getSourceDirAbsPath(nil)
+
+	return &templateData{
+		Arch:         runtime.GOARCH,
+		Args:         os.Args,
+		CacheDir:     c.CacheDirAbsPath,
+		ConfigFile:   c.configFileAbsPath,
+		Executable:   chezmoi.NewAbsPath(executable),
+		FQDNHostname: fqdnHostname,
+		GID:          gid,
+		Group:        group,
+		HomeDir:      c.homeDirAbsPath,
+		Hostname:     hostname,
+		Kernel:       kernel,
+		OS:           runtime.GOOS,
+		OSRelease:    osRelease,
+		SourceDir:    sourceDirAbsPath,
+		UID:          uid,
+		Username:     username,
+		Version: map[string]any{
+			"builtBy": c.versionInfo.BuiltBy,
+			"commit":  c.versionInfo.Commit,
+			"date":    c.versionInfo.Date,
+			"version": c.versionInfo.Version,
+		},
+		WindowsVersion: windowsVersion,
+		WorkingTree:    c.WorkingTreeAbsPath,
+	}
+}
+
 // readConfig reads the config file, if it exists.
 func (c *Config) readConfig() error {
 	switch err := c.decodeConfigFile(c.configFileAbsPath, &c.ConfigFile); {
@@ -1919,19 +2110,6 @@ func (c *Config) sourceAbsPaths(sourceState *chezmoi.SourceState, args []string)
 	return sourceAbsPaths, nil
 }
 
-// sourceDirAbsPath returns the source directory, using .chezmoiroot if it
-// exists.
-func (c *Config) sourceDirAbsPath() (chezmoi.AbsPath, error) {
-	switch data, err := c.sourceSystem.ReadFile(c.SourceDirAbsPath.JoinString(chezmoi.RootName)); {
-	case errors.Is(err, fs.ErrNotExist):
-		return c.SourceDirAbsPath, nil
-	case err != nil:
-		return chezmoi.EmptyAbsPath, err
-	default:
-		return c.SourceDirAbsPath.JoinString(string(bytes.TrimSpace(data))), nil
-	}
-}
-
 func (c *Config) targetRelPath(absPath chezmoi.AbsPath) (chezmoi.RelPath, error) {
 	relPath, err := absPath.TrimDirPrefix(c.DestDirAbsPath)
 	var notInAbsDirError *chezmoi.NotInAbsDirError
