@@ -22,6 +22,14 @@ const (
 
 var onepasswordVersionRx = regexp.MustCompile(`^(\d+\.\d+\.\d+\S*)`)
 
+type onepasswordAccount struct {
+	URL         string `json:"url"`
+	Email       string `json:"email"`
+	UserUUID    string `json:"user_uuid"`    //nolint:tagliatelle
+	AccountUUID string `json:"account_uuid"` //nolint:tagliatelle
+	Shorthand   string `json:"shorthand"`
+}
+
 type onepasswordConfig struct {
 	Command       string
 	Prompt        bool
@@ -30,6 +38,8 @@ type onepasswordConfig struct {
 	environFunc   func() []string
 	outputCache   map[string][]byte
 	sessionTokens map[string]string
+	accountMap    map[string]string
+	accountMapErr error
 }
 
 type onepasswordArgs struct {
@@ -70,7 +80,7 @@ func (c *Config) onepasswordTemplateFunc(userArgs ...string) map[string]any {
 		})
 	}
 
-	args, err := newOnepasswordArgs(baseArgs, userArgs)
+	args, err := c.newOnepasswordArgs(baseArgs, userArgs)
 	if err != nil {
 		panic(err)
 	}
@@ -155,7 +165,7 @@ func (c *Config) onepasswordDocumentTemplateFunc(userArgs ...string) string {
 		})
 	}
 
-	args, err := newOnepasswordArgs(baseArgs, userArgs)
+	args, err := c.newOnepasswordArgs(baseArgs, userArgs)
 	if err != nil {
 		panic(err)
 	}
@@ -287,7 +297,7 @@ func (c *Config) onepasswordGetOrRefreshSessionToken(args *onepasswordArgs) (str
 }
 
 func (c *Config) onepasswordItemV1(userArgs []string) (*onepasswordItemV1, error) {
-	args, err := newOnepasswordArgs([]string{"get", "item"}, userArgs)
+	args, err := c.newOnepasswordArgs([]string{"get", "item"}, userArgs)
 	if err != nil {
 		return nil, err
 	}
@@ -305,7 +315,7 @@ func (c *Config) onepasswordItemV1(userArgs []string) (*onepasswordItemV1, error
 }
 
 func (c *Config) onepasswordItemV2(userArgs []string) (*onepasswordItemV2, error) {
-	args, err := newOnepasswordArgs([]string{"item", "get", "--format", "json"}, userArgs)
+	args, err := c.newOnepasswordArgs([]string{"item", "get", "--format", "json"}, userArgs)
 	if err != nil {
 		return nil, err
 	}
@@ -359,11 +369,12 @@ func (c *Config) onepasswordReadTemplateFunc(url string, args ...string) string
 	onepasswordArgs := &onepasswordArgs{
 		args: []string{"read", url},
 	}
+
 	switch len(args) {
 	case 0:
 		// Do nothing.
 	case 1:
-		onepasswordArgs.args = append(onepasswordArgs.args, "--account", args[0])
+		onepasswordArgs.args = append(onepasswordArgs.args, "--account", c.onepasswordAccount(args[0]))
 	default:
 		panic(fmt.Errorf("expected 1 or 2 arguments, got %d", len(args)))
 	}
@@ -375,6 +386,61 @@ func (c *Config) onepasswordReadTemplateFunc(url string, args ...string) string
 	return string(output)
 }
 
+func (c *Config) onepasswordAccount(key string) string {
+	version, err := c.onepasswordVersion()
+	if err != nil {
+		panic(err)
+	}
+
+	// Account listing does not affect version 1
+	if version.Major == 1 {
+		return key
+	}
+
+	accounts, err := c.onepasswordAccounts()
+	if err != nil {
+		panic(err)
+	}
+
+	if account, exists := accounts[key]; exists {
+		return account
+	}
+
+	panic(fmt.Errorf("no 1Password account found matching %s", key))
+}
+
+// onepasswordAccounts returns a map of keys to unique account UUIDs.
+func (c *Config) onepasswordAccounts() (map[string]string, error) {
+	if c.Onepassword.accountMap != nil || c.Onepassword.accountMapErr != nil {
+		return c.Onepassword.accountMap, c.Onepassword.accountMapErr
+	}
+
+	if version, err := c.onepasswordVersion(); err != nil {
+		if version.Major == 1 {
+			return make(map[string]string), nil
+		}
+	}
+
+	args := &onepasswordArgs{
+		args: []string{"account", "list", "--format=json"},
+	}
+
+	output, err := c.onepasswordOutput(args, withoutSessionToken)
+	if err != nil {
+		c.Onepassword.accountMapErr = err
+		return nil, c.Onepassword.accountMapErr
+	}
+
+	var accounts []onepasswordAccount
+	if err := json.Unmarshal(output, &accounts); err != nil {
+		c.Onepassword.accountMapErr = err
+		return nil, c.Onepassword.accountMapErr
+	}
+
+	c.Onepassword.accountMap = onepasswordAccountMap(accounts)
+	return c.Onepassword.accountMap, c.Onepassword.accountMapErr
+}
+
 func (c *Config) onepasswordVersion() (*semver.Version, error) {
 	if c.Onepassword.version != nil || c.Onepassword.versionErr != nil {
 		return c.Onepassword.version, c.Onepassword.versionErr
@@ -410,7 +476,7 @@ func (c *Config) onepasswordVersion() (*semver.Version, error) {
 	return c.Onepassword.version, c.Onepassword.versionErr
 }
 
-func newOnepasswordArgs(baseArgs, userArgs []string) (*onepasswordArgs, error) {
+func (c *Config) newOnepasswordArgs(baseArgs, userArgs []string) (*onepasswordArgs, error) {
 	if len(userArgs) < 1 || 3 < len(userArgs) {
 		return nil, fmt.Errorf("expected 1, 2, or 3 arguments, got %d", len(userArgs))
 	}
@@ -418,19 +484,65 @@ func newOnepasswordArgs(baseArgs, userArgs []string) (*onepasswordArgs, error) {
 	a := &onepasswordArgs{
 		args: baseArgs,
 	}
+
 	a.item = userArgs[0]
 	a.args = append(a.args, a.item)
+
 	if len(userArgs) > 1 && userArgs[1] != "" {
 		a.vault = userArgs[1]
 		a.args = append(a.args, "--vault", a.vault)
 	}
+
 	if len(userArgs) > 2 && userArgs[2] != "" {
-		a.account = userArgs[2]
+		a.account = c.onepasswordAccount(userArgs[2])
 		a.args = append(a.args, "--account", a.account)
 	}
 	return a, nil
 }
 
+// onepasswordAccountMap returns a map of unique IDs to account UUIDs.
+func onepasswordAccountMap(accounts []onepasswordAccount) map[string]string {
+	// Build a map of keys to account UUIDs.
+	accountsMap := make(map[string][]string)
+	for _, account := range accounts {
+		keys := []string{
+			account.URL,
+			account.Email,
+			account.UserUUID,
+			account.AccountUUID,
+			account.Shorthand,
+		}
+
+		accountName, _, accountNameOk := strings.Cut(account.URL, ".")
+		if accountNameOk {
+			keys = append(keys, accountName)
+		}
+
+		emailName, _, emailNameOk := strings.Cut(account.Email, "@")
+		if emailNameOk {
+			keys = append(keys, emailName, emailName+"@"+account.URL)
+		}
+
+		if accountNameOk && emailNameOk {
+			keys = append(keys, emailName+"@"+accountName)
+		}
+
+		for _, key := range keys {
+			accountsMap[key] = append(accountsMap[key], account.AccountUUID)
+		}
+	}
+
+	// Select unique, non-empty keys.
+	accountMap := make(map[string]string)
+	for key, values := range accountsMap {
+		if key != "" && len(values) == 1 {
+			accountMap[key] = values[0]
+		}
+	}
+
+	return accountMap
+}
+
 // onepasswordUniqueSessionToken will look for any session tokens in the
 // environment. If it finds exactly one then it will return it.
 func onepasswordUniqueSessionToken(environ []string) string {
