@@ -21,7 +21,7 @@ import (
 	"strings"
 
 	"github.com/coreos/go-semver/semver"
-	"github.com/google/go-github/v45/github"
+	"github.com/google/go-github/v47/github"
 	"github.com/spf13/cobra"
 	vfs "github.com/twpayne/go-vfs/v4"
 
@@ -539,7 +539,7 @@ func getUpgradeMethod(fileSystem vfs.Stater, executableAbsPath chezmoi.AbsPath)
 
 // getPackageType returns the distributions package type based on is OS release.
 func getPackageType(system chezmoi.System) (string, error) {
-	osRelease, err := chezmoi.OSRelease(system)
+	osRelease, err := chezmoi.OSRelease(system.UnderlyingFS())
 	if err != nil {
 		return packageTypeNone, err
 	}
